@@ -6,9 +6,9 @@ use Data::Dumper;
 
 extends 'Catalyst::Controller';
 
-sub root :Chained(/) PathPart('account') CaptureArgs(0) { }
+sub root :Chained('/') PathPart('account') CaptureArgs(0) { }
 
-  sub one :Chained(root) PathPart('one') Args(0) Does(RequestModel) RequestModel(AccountRequest) {
+  sub one :Chained(root) PathPart('one') Args(0) Does(RequestModel) BodyModel(AccountRequest) {
     my ($self, $c, $request) = @_;
     $c->res->body(Dumper $request->nested_params);
   }
